{% extends "./chart.html" %}

<script>
    {% block apexcharts_additional_options %}
        apex_options.legend = {
            formatter: function (seriesName, opts) {
                return [seriesName, ' : <span class="format-number">' + opts.w.globals.series[opts.seriesIndex] + '</span>']
            }
        }
    {% endblock %}
</script>