<div class="modal fade" id="listing-popup" tabindex="-1" role="dialog" aria-hidden="true">
    <div class="modal-dialog modal-dialog-centered" role="document">
        <div class="modal-content">
            <div class="modal-header">
                <h5 class="modal-title">{{ object }}</h5>
                <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
            </div>
            <div class="modal-body">

                <table class="object-popup">
                    {% for row in rows %}
                        <tr>
                            {% for label, value in row %}
                                {% if label == "== section ==" %}
                                    <td class="section" colspan="{{ full_colspan }}"><span>{{ value|safe }}</span></td>
                                {% else %}
                                    <th>{{ label }}{{ label_ending }}</th> {{ value|safe }}
                                {% endif %}
                            {% endfor %}
                        </tr>
                    {% endfor %}
                </table>

            </div>
            <div class="modal-footer">
                <button type="button" class="btn btn-primary" data-bs-dismiss="modal">Close</button>
            </div>
        </div>
    </div>
</div>