{% load static %}
{% load django_listing %}
{% spaceless %}
    {% if AUTO_DECLARE_CSS %}
        {% if need_media_for.datetimepicker and DATETIMEPICKER_CSS_URL %}
            <link rel="stylesheet"
                  href="{{ DATETIMEPICKER_CSS_URL }}?{{ STATIC_FILES_VERSION }}"/>
        {% endif %}
        {% if need_media_for.dropzone and DROPZONE_CSS_URL %}
            <link rel="stylesheet"
                  href="{{ DROPZONE_CSS_URL }}?{{ STATIC_FILES_VERSION }}"/>
        {% endif %}
        {% if need_media_for.dual_listbox and DUAL_LISTBOX_CSS_URL %}
            <link rel="stylesheet"
                  href="{{ DUAL_LISTBOX_CSS_URL }}?{{ STATIC_FILES_VERSION }}"/>
        {% endif %}
        <link rel="stylesheet"
              href="{% static "django_listing" %}/{{ THEME }}/css/django_listing.min.css?{{ STATIC_FILES_VERSION }}"/>
    {% endif %}

    {% if need_media_for.autocomplete and AUTOCOMPLETE_CSS_URLS %}
        {% if AUTO_DECLARE_CSS %}
            {% for url in AUTOCOMPLETE_CSS_URLS %}
                <link rel="stylesheet" href="{{ url }}?{{ STATIC_FILES_VERSION }}"/>
            {% endfor %}
        {% endif %}
    {% endif %}

    {% header_snippets %}
{% endspaceless %}